package Devel::Peek::Capture;

use strict;
use vars qw($VERSION);
use base qw(Exporter);
use Devel::Peek ();
use IO::CaptureOutput;

$VERSION = '0.01';

@Devel::Peek::Capture::EXPORT    = qw(Dump);
@Devel::Peek::Capture::EXPORT_OK = qw(is_shared);


# Capture STDERR

sub Dump {
    my ($stdout, $stderr);

    local $^W = undef;
    local *IO::CaptureOutput::capture = \&_capture;

    IO::CaptureOutput::capture(sub {
        eval q{ Devel::Peek::Dump(@_) };
        if($@){ print $@; }
    }, \$stdout, \$stderr, @_);

    if($stdout){
        _die($stdout);
    }

    return $stderr;
}


# The original IO::CaptureOutput::capture doesn't pass @_.

sub _capture {
    my $code   = shift;
    my $output = shift;
    my $error  = shift;

    for ($output, $error) {
        $_ = \do { my $s; $s = ''} unless ref $_;
        $$_ = '' unless defined($$_);
    }

    my $capture_out = new IO::CaptureOutput::_proxy('STDOUT', $output);
    my $capture_err = new IO::CaptureOutput::_proxy('STDERR', $error);

    $code->(@_);
}


sub _die {
    my $stdout = shift;
    my ($filename, $line) = ( caller(1) )[1,2];
    $stdout =~ s/ at \(eval.+$/ at $filename line $line/;
    die $stdout;
}


#
# Functions?
#

sub is_shared {
    my $dumped = Dump(@_);

    for my $line (split/\n/,$dumped){
        return 1 if($line =~ /^\s+MG_TYPE = PERL_MAGIC_shared_/);
    }

    return 0;
}

1;
__END__

=head1 NAME

Devel::Peek::Capture - capture Devel::Peek's output

=head1 SYNOPSIS

 use Devel::Peek::Capture;
 
 my $foo    = {};
 my $dumped = Dump($foo);
 
 print $dumped; # from Devel::Peek


=head1 DESCRIPTION

L<Devel::Peek> is a very useful module but its message is printed out
to STDERR. This module can caputure the Devel::Peek output.


=head1 EXPORT

C<Dump> by default. It works as same as Devel::Peek's one
but returns string.

You can export C<is_shared> which tests if passed thing (sv) is
C<threads::share>d.


=head1 See Also

L<Devel::Peek>

L<IO::CaptureOutput>

=head1 AUTHOR

 Makamaka Hannyaharamitu makamaka[at]cpan.org

=head1 COPYRIGHT

This program is free software; you can redistribute
it and/or modify it under the same terms as Perl itself.

The full text of the license can be found in the
LICENSE file included with this module.


=cut


